library flutter_gh_fetch;

import 'package:flutter_gh_fetch/models/repo.model.dart';
import 'package:flutter_gh_fetch/models/user.model.dart';
import 'package:graphql/client.dart';

/// Minimal package for fetching GitHub user info and paginated list
/// of repositories. This library uses GitHub GraphQL API in order to
/// fetch data related to the user and their repositories. The
/// authentication is conducted through the use of Personal Access Token,
/// which you need to provide as a constructor argument.
class FlutterGHFetch {
  static const String _GITHUB_API_URL = 'https://api.github.com/graphql';

  final String _accessToken;
  GraphQLClient _client;

  /// Constructor -> provide personal access token.
  FlutterGHFetch(this._accessToken);

  /// Call this method to initialize the undelying GraphQL client. You
  /// need to call this method first, before any other query!
  Future<void> init() async {
    final HttpLink httpLink = HttpLink(
      uri: _GITHUB_API_URL,
    );
    final AuthLink authLink = AuthLink(
      getToken: () async => 'Bearer $_accessToken',
    );
    _client = GraphQLClient(
      cache: InMemoryCache(),
      link: authLink.concat(httpLink),
    );
  }

  /// Query user data and return [UserModel] instance.
  Future<UserModel> queryUserData() async {
    final QueryResult result = await _client.query(
      QueryOptions(
        documentNode: gql(
          _userDataQuery(),
        ),
      ),
    );
    return UserModel.from(result.data['viewer']);
  }

  /// Query repository data and return a list of [RepoModel]
  /// instances. You need to provide the amount of repositories
  /// you would like to query with [first] parameter, while [after]
  /// and [filterPrivate] are completely optional. With [after] you
  /// achieve pagination, so first [first] repositories after cursor
  /// [after]. With [filterPrivate] being set to true, you only
  /// query for public repositories.
  Future<List<RepoModel>> queryRepositoryData(
    int first, {
    String after,
    bool filterPrivate = false,
  }) async {
    final QueryResult result = await _client.query(
      QueryOptions(
        documentNode: gql(
          _repositoryDataQuery(first, cursor: after),
        ),
      ),
    );
    return filterPrivate != null && filterPrivate
        ? result.data['viewer']['repositories']['edges']
            .map<RepoModel>(
              (repo) => RepoModel.from(repo),
            )
            .where((repo) => !repo.isPrivate)
            .toList()
        : result.data['viewer']['repositories']['edges']
            .map<RepoModel>(
              (repo) => RepoModel.from(repo),
            )
            .toList();
  }

  String _userDataQuery() {
    return r'''
      query {
        viewer{
          avatarUrl
          name
          login
          bio
          company
          location
          websiteUrl
        }
      }
    ''';
  }

  String _repositoryDataQuery(int first, {String cursor}) {
    final String cursorVariable = cursor != null ? ', after: "$cursor"' : '';
    return '''
      query {
        viewer {
          repositories(first: $first${cursorVariable.toString()}) {
            edges {
              cursor
              node {
                name
                nameWithOwner
                description
                url
                isPrivate
                primaryLanguage {
                  name
                  color
                }
              }
            }
          }
        }
      }
    ''';
  }
}
